import { PageContainer } from '@ant-design/pro-layout';
import { history } from 'umi'
import React, { useState, useEffect, Fragment } from 'react'
import { Form, Input, Select, InputNumber, Card, Button, Radio, Space, notification, Result, Alert, Tooltip, message } from 'antd'
import { getQueryString } from '@/utils/Url';
import { _minerType, _minerPriceName } from '@/utils/type';
import RadioGroupInput from '@/components/Custom/RadioGroupInput/RadioGroupInput';
import { createMiner, updateMiner, getDetail } from '../services'
import { objFilter } from '@/utils/base';
import { ExclamationCircleOutlined } from '@ant-design/icons';

const Option = Select.Option

const formItemLayout = {
  labelCol: {
    xs: { span: 24 },
    sm: { span: 5 },
  },
  wrapperCol: {
    xs: { span: 24 },
    sm: { span: 12 },
  },
};

export default function AddMachine() {

  const [form] = Form.useForm();
  const tid = getQueryString('id')
  const [minerLimit, setMinerLimit] = useState<number | undefined>(1)
  const [perUserLimit, setPerUserLimit] = useState<number | undefined>(1)
  const [submitLoading, setSubmitLoading] = useState<boolean>(false)
  const [finished, setFinished] = useState(false)
  const [loading, setLoading] = useState(false)
  const [minerLimitRadio, setMinerLimitRadio] = useState('0')
  const [perUserLimitRadio, setPerUserLimitRadio] = useState('0')
  const [errMsg, setErrMsg] = useState()
  const [, setInfo] = useState<any>({})
  const [, updateForm] = useState(false)

  useEffect(() => {
    initFun()
  }, [])

  const initFun = () => {
    if (tid) {
      fetchDetail(tid)
    }
  }

  //获取详情
  const fetchDetail = async (tid: any) => {
    setLoading(true)
    const res: any = await getDetail({ tid })
    if (res.success) {
      setLoading(false)
      const info = res.data || {}
      const fields: any = {
        name: info.name,
        type: info.type,
        outputCycle: info.outputCycle,
        dayYield: info.dayYield,
        status: info.status,
        sortNo: info.sortNo,
      }
      if (info.outputRule) {
        fields.daibi = info.outputRule.daibi
        fields.quanyizhi = info.outputRule.quanyizhi
      }
      if (info.description) {
        fields.power = info.description.power
      }
      if (info.minerLimit === 0) {
        setMinerLimitRadio('0')
      } else if (info.minerLimit) {
        setMinerLimit(info.minerLimit)
        setMinerLimitRadio('1')
      }
      if (info.perUserLimit === 0) {
        setPerUserLimitRadio('0')
      } else if (info.perUserLimit) {
        setPerUserLimit(info.perUserLimit)
        setPerUserLimitRadio('1')
      }
      await form.setFieldsValue(fields)
      await form.setFieldsValue({ price: info.price })
      setInfo(info)
    } else {
      setErrMsg(res.msg)
    }
  }

  //提交
  const onSubmit = () => {
    form.validateFields().then(async (values) => {
      console.log(values)
      const params: any = {
        name: values.name,
        type: values.type,
        price: values.price,
        outputCycle: values.outputCycle,
        dayYield: values.dayYield,
        minerLimit: minerLimitRadio === '0' ? 0 : minerLimit,
        perUserLimit: perUserLimitRadio === '0' ? 0 : perUserLimit,
        status: values.status,
        sortNo: values.sortNo,
        tid
      }
      //矿机描述
      params.description = JSON.stringify({
        power: values.power
      })
      //产出规则
      if (values.daibi + values.quanyizhi > 100) {
        message.error('产出规则比例总和不能大于100%')
        return
      }
      params.outputRule = JSON.stringify({
        daibi: values.daibi,
        quanyizhi: values.quanyizhi
      })
      objFilter(params)
      console.log('提交', params)
      if (submitLoading) return
      await setSubmitLoading(true)
      const res = tid ? await updateMiner(params) : await createMiner(params)
      setSubmitLoading(false)
      if (res.success) {
        notification.success({
          message: tid ? '编辑成功' : '添加成功'
        })
        setFinished(true)
      }
    })
  }

  const testLog = () => {

  }

  //重新添加
  const addAgain = () => {
    setFinished(false)
    setMinerLimit(1)
    setPerUserLimit(1)
    form.resetFields()
  }

  //错误重试
  const openAgain = () => {
    fetchDetail(tid)
  }

  return (
    <Fragment>
      {errMsg && (
        <Result
          status="error"
          title={errMsg}
          extra={[
            <Button type="primary" key="again" onClick={openAgain}>
              重试
          </Button>
          ]}
        />
      )}
      {finished && !errMsg && (
        <Result
          status="success"
          title={tid ? '编辑成功' : '添加成功'}
          extra={[
            <Button type="primary" key="add" onClick={addAgain}>
              继续添加
        </Button>,
            <Button key="buy" onClick={() => history.goBack()}>返回</Button>,
            <Button hidden={!tid} type="primary" key="update" onClick={() => location.reload()}>
              重新编辑
        </Button>
          ]}
        />
      )}
      {!finished && !errMsg && (
        <PageContainer
          onBack={() => history.goBack()}
          footer={[
            <Button type="primary" key={1} loading={submitLoading} onClick={onSubmit}>{tid ? '确定编辑' : '确定添加'}</Button>
          ]}
          style={{ display: finished ? 'none' : 'block' }}
        >
          <Form
            form={form}
            {...formItemLayout}
            onValuesChange={() => updateForm(mark => !mark)}
          >
            <Space direction="vertical" size="large" style={{ width: '100%' }}>
              <Card loading={loading} title={<span onClick={testLog}>基础信息</span>}>
                <Form.Item
                  label="类型"
                  name="type"
                  rules={[{ message: '请选择矿机类型', required: true }]}
                >
                  <Select placeholder="请选择矿机类型">
                    {Object.keys(_minerType).map(key => (
                      <Option key={key} value={Number(key)}>{_minerType[key]}</Option>
                    ))}
                  </Select>
                </Form.Item>

                <Form.Item
                  label="名称"
                  name="name"
                  rules={[{ message: '请输入矿机名称', required: true }]}
                >
                  <Input placeholder="请输入矿机名称" />
                </Form.Item>

                <Form.Item
                  label={
                    <Fragment>
                      <span>排序值</span>
                      <Tooltip placement="top" title="数值大位置越靠前，如果两个排序数值相同，则创建时间越新越靠前">
                        <ExclamationCircleOutlined style={{ fontSize: 12, color: '#999', cursor: 'pointer' }} />
                      </Tooltip>
                    </Fragment>
                  }
                  name="sortNo"

                  rules={[{ message: '请输入排序', required: true }]}
                >
                  <InputNumber style={{ width: '100%' }} placeholder="请输入排序" min={1} precision={0} step={1} />
                </Form.Item>

                {form.getFieldValue('type') && (
                  <Form.Item
                    label={`购买需支付(${_minerPriceName[form.getFieldValue('type')] || ''})`}
                    name="price"
                    rules={[{ message: '请输入购买价格', required: true }]}
                  >
                    <InputNumber
                      style={{ width: '100%' }}
                      placeholder={`${_minerPriceName[form.getFieldValue('type')] || ''}`}
                      min={0.01}
                      step={1}
                      formatter={value => `${value}`.replace(/\B(?=(\d{3})+(?!\d))/g, ',')}
                    />
                  </Form.Item>
                )}

                <Form.Item
                  label={
                    <Fragment>
                      <span>产出周期</span>
                      <Tooltip placement="top" title="每周计为7天，每月计为30天">
                        <ExclamationCircleOutlined style={{ fontSize: 12, color: '#999', cursor: 'pointer' }} />
                      </Tooltip>
                    </Fragment>
                  }
                  name="outputCycle"
                  rules={[{ message: '请输入产出周期', required: true }]}
                >
                  <InputNumber
                    style={{ width: '100%' }}
                    placeholder="请输入产出周期"
                    min={1}
                    precision={0}
                    step={1}
                    formatter={value => `${value} 天`}
                  />
                </Form.Item>

                <Form.Item label="每天产量" required>
                  <Form.Item
                    noStyle
                    name="dayYield"
                    rules={[{ message: '请输入每天产量', required: true }]}
                  >
                    <InputNumber
                      style={{ width: 'calc(50%)' }}
                      placeholder="请输入每天产量"
                      min={0}
                      step={1}
                    />
                  </Form.Item>
                  <span style={{ marginLeft: 20 }}>月产量(30天)：{(form.getFieldValue('dayYield') || 0) * 30}</span>
                </Form.Item>

                <Form.Item
                  label="矿机限购数量(台)"
                  required
                >
                  <RadioGroupInput
                    setInputValue={(value: number) => setMinerLimit(value)}
                    inputValue={minerLimit}
                    placeholder="请输入矿机限购数量"
                    radioValue={minerLimitRadio}
                    setRadioValue={(minerLimitRadio: string) => setMinerLimitRadio(minerLimitRadio)}
                  />
                </Form.Item>

                <Form.Item
                  label="每人限购数量(台)"
                  required
                >
                  <RadioGroupInput
                    setInputValue={(value: number) => setPerUserLimit(value)}
                    inputValue={perUserLimit}
                    placeholder="请输入每人限购数量"
                    radioValue={perUserLimitRadio}
                    setRadioValue={(perUserLimitRadio: string) => setPerUserLimitRadio(perUserLimitRadio)}
                  />
                </Form.Item>

                <Form.Item
                  label="矿机状态"
                  name="status"
                  rules={[{ message: '请选择矿机状态', required: true }]}
                >
                  <Radio.Group
                    options={[
                      { label: '启用', value: 1 },
                      { label: '停用', value: 0 }
                    ]}
                  />
                </Form.Item>
              </Card>
            </Space>
          </Form>
        </PageContainer >
      )}

    </Fragment>
  )
}