import { PageContainer } from '@ant-design/pro-layout';
import { SearchOutlined, SyncOutlined, PlusOutlined, ExclamationCircleOutlined } from '@ant-design/icons';
import ProTable, { ProColumns, ActionType } from '@ant-design/pro-table';
import React, { useRef, Fragment, useState, useEffect } from 'react'
import { Button, Divider, Card, Popconfirm, notification, Tooltip } from 'antd'
import { getMachinePageList, IMachinePageList, updateMiner } from '../services'
import * as Format from '@/utils/Format'
import { objFilter } from '@/utils/base';
import { _minerType, _minerPriceName } from '@/utils/type';
import { FormInstance } from 'antd/lib/form';
import Search from './components/Search'
import { history } from 'umi';

export default () => {

  const searchFormRef = useRef<FormInstance>()
  const actionRef = useRef<ActionType>();
  const [pagination, setPagination] = useState(() => {
    let storageParams = sessionStorage.getMachinePageList;
    try {
      storageParams = JSON.parse(storageParams)
    } catch (error) {
      storageParams = {}
    }
    return { current: storageParams.pageIndex || 1, pageSize: storageParams.pageSize || 20, total: 0 }
  })
  const [loading, setLoading] = useState(false)

  useEffect(() => {
    initSearchForm()
  }, [])


  const columns: ProColumns<any>[] = [
    {
      title: '名称',
      dataIndex: 'name'
    },
    {
      title: (
        <Fragment>
          <span>排序值</span>
          <Tooltip placement="top" title="数值大位置越靠前，如果两个排序数值相同，则创建时间越新越靠前">
            <ExclamationCircleOutlined style={{ fontSize: 12, color: '#999', cursor: 'pointer' }} />
          </Tooltip>
        </Fragment>
      ),
      dataIndex: 'sortNo',
    },
    {
      title: '状态',
      dataIndex: 'status',
      valueEnum: {
        0: { text: '停用', status: 'Error' },
        1: { text: '启用', status: 'Success' },
      },
    },
    {
      title: '类型',
      dataIndex: 'type',
      render: (type: any) => _minerType[type]
    },
    {
      title: '购买需支付',
      dataIndex: 'price',
      render: (price: any, record: any) => `${price.toString().replace(/\B(?=(\d{3})+(?!\d))/g, ',')}(${_minerPriceName[record.type]})`
    },
    {
      title: '算力值',
      dataIndex: 'description',
      render: (description: any) => description.power
    },
    {
      title: '产量',
      dataIndex: 'monthYield',
      render: (monthYield: any, record: any) => (
        <Fragment>
          <div>{`${record.dayYield} /日`}</div>
          <div>{`${record.monthYield} /月`}</div>
        </Fragment>
      )
    },
    {
      title: '发币周期(天)',
      dataIndex: 'outputCycle',
    },
    {
      title: '发币币种及比例',
      dataIndex: 'outputRule',
      render: (outputRule: any) => {
        return (
          <Fragment>
            {outputRule.daibi && <div>WOS({`${outputRule.daibi}%`})</div>}
            {outputRule.quanyizhi && <div>共享值({`${outputRule.quanyizhi}%`})</div>}
          </Fragment>
        )
      }
    },
    {
      title: '剩余数量(台)',
      dataIndex: 'stockQuantity',
      render: (stockQuantity: any) => stockQuantity === 0 ? '不限' : stockQuantity
    },
    {
      title: '限购数量(人/台)',
      dataIndex: 'perUserLimit',
      render: (perUserLimit: any) => perUserLimit === 0 ? '不限' : perUserLimit
    },
    {
      title: '创建时间',
      dataIndex: 'createTime',
      render: createTime => Format.date(createTime, 'yyyy-MM-dd HH:mm')
    },
    {
      title: '操作',
      dataIndex: 'option',
      valueType: 'option',
      render: (_, record) => (
        <>
          <a onClick={() => history.push('/machine/update?id=' + record.tid)}>编辑</a>
          {/* <Divider type="vertical" />
          <a>删除</a> */}
          <Divider type="vertical" />
          {record.status === 1 && (
            <Popconfirm
              title="确定禁用吗？"
              onConfirm={() => handleStatus(record.tid, 0)}
            >
              <a>禁用</a>
            </Popconfirm>
          )}
          {record.status === 0 && (
            <Popconfirm
              title="确定启用吗？"
              onConfirm={() => handleStatus(record.tid, 1)}
            >
              <a>启用</a>
            </Popconfirm>
          )}
        </>
      ),
    },
  ];

  //禁用-启用
  const handleStatus = async (tid: string, status: number) => {
    const res = await updateMiner({ tid, status })
    if (res.success) {
      notification.success({
        message: `${status === 0 ? '禁用' : '启用'}成功`,
      });
      actionRef.current?.reload()
    }
  }

  //预设搜索表单数据
  const initSearchForm = () => {
    let storageParams = sessionStorage.getMachinePageList;
    try {
      storageParams = JSON.parse(storageParams)
    } catch (error) {
      storageParams = {}
    }
    const formFields = {
      name: storageParams.name,
      status: storageParams.status,
      type: storageParams.type,
    };
    // if (storageParams.startDate && storageParams.endDate) {
    //   formFields.time = [moment(params.startDate), moment(params.endDate)];
    // }
    searchFormRef?.current?.setFieldsValue(formFields);
  }

  //获取列表数据
  const fetchList = async (params: any, sort: any, filter: any) => {
    console.log('params: ', params)
    const formData: any = searchFormRef.current?.getFieldsValue()
    const fetchParams: IMachinePageList = {
      pageIndex: params.current,
      pageSize: params.pageSize,
      name: formData.name,
      type: formData.type,
      status: formData.status
    }
    // console.log('fetchParams: ', fetchParams)
    //存储搜索项目
    sessionStorage.setItem('getMachinePageList', JSON.stringify(fetchParams))
    //删除未搜索项目
    objFilter(fetchParams)
    setLoading(true)
    const res = await getMachinePageList(fetchParams)
    setLoading(false)
    if (res.success) {
      await setPagination((pagination: any) => {
        pagination.total = res.data.total
        pagination.pageSize = res.data.pageSize
        return { ...pagination }
      })
    }
    return {
      data: res.data.list,
      total: res.data.total,
      success: res.success
    }
  }

  //翻页
  const handlePage = async (current: number, pageSize?: number | undefined) => {
    await setPagination(pagination => {
      return {
        ...pagination,
        current,
        pageSize: pageSize || 20
      }
    })
    actionRef.current?.reload()
  }

  //测试数据
  const testLog = () => {

  }

  const AddOption = () => {
    return (
      <Button type="primary" onClick={() => history.push('/machine/add')} icon={<PlusOutlined />}>添加矿机</Button>
    )
  }

  //点击查询
  const onSearch = async () => {
    await setPagination(pagination => (
      {
        ...pagination,
        current: 1
      }
    ))
    actionRef.current?.reload()
  }

  return (
    <PageContainer extra={<AddOption />}>
      <Card title={<span onClick={testLog}>矿机列表</span>}>
        <Search
          formRef={searchFormRef}
        />
        <ProTable
          bordered
          actionRef={actionRef}
          rowKey="tid"
          toolBarRender={() => [
            <Button style={{ marginLeft: 20 }} icon={<SyncOutlined />} onClick={() => searchFormRef.current?.resetFields()}>
              重置
            </Button>,
            <Button
              type="primary"
              loading={loading}
              icon={<SearchOutlined />}
              onClick={onSearch}
            >
              查询
           </Button>
          ]}
          request={fetchList}
          columns={columns}
          //搜索配置
          search={false}
          pagination={{
            ...pagination,
            onChange: handlePage
          }}
        />
      </Card>
    </PageContainer>
  )
}