/**
 * request 网络请求工具
 * 更详细的 api 文档: https://github.com/umijs/umi-request
 */
import { extend } from 'umi-request';
import { stringify } from 'querystring';
import Cookie from './Cookie';
import Security from './Security'
import { Base64 } from 'js-base64'
import { getQueryString } from '@/utils/Url'
import { notification } from 'antd';

const codeMessage = {
  200: '服务器成功返回请求的数据。',
  201: '新建或修改数据成功。',
  202: '一个请求已经进入后台排队（异步任务）。',
  204: '删除数据成功。',
  400: '发出的请求有错误，服务器没有进行新建或修改数据的操作。',
  401: '用户没有权限（令牌、用户名、密码错误）。',
  403: '用户得到授权，但是访问是被禁止的。',
  404: '发出的请求针对的是不存在的记录，服务器没有进行操作。',
  406: '请求的格式不可得。',
  410: '请求的资源被永久删除，且不会再得到的。',
  422: '当创建一个对象时，发生一个验证错误。',
  500: '服务器发生错误，请检查服务器。',
  502: '网关错误。',
  503: '服务不可用，服务器暂时过载或维护。',
  504: '网关超时。',
};

/**
 * 异常处理程序
 */
const errorHandler = error => {
  const { response } = error;

  if (response && response.status) {
    const errorText = codeMessage[response.status] || response.statusText;
    const { status, url } = response;
    // notification.error({
    //   message: `请求错误 ${status}: ${url}`,
    //   description: errorText,
    // });
  } else if (!response) {
    // notification.error({
    //   description: '您的网络发生异常，无法连接服务器',
    //   message: '网络异常',
    // });
  }

  return response;
};

/**
 * 配置request请求时的默认参数
 */
const request = extend({
  errorHandler,
  // 默认错误处理
  credentials: 'include', // 默认请求是否带上cookie
});

/**
 * 请求头处理
 */
const getHeader = () => {
  // 渠道号
  const ci = getQueryString('c') || getQueryString('ci') || '0'
  // 通用参数
  const commonParams = {
    a: 11,
    av: '1.0.0',
    c: 4,
    ci: ci,
    di: '1a00bf0e978c458f7620b390714b26cc',
    lat: '',
    lng: '',
    pm: '',
    pn: 'com.starbuds.h5',
    st: 1,
    sv: '',
    t: Cookie.get('_xy_music', { path: '/' }) || '',
    ts: new Date().getTime()
  };

  let toSign = '';
  Object.keys(commonParams).map((key) => {
    toSign += '&' + key + '=' + commonParams[key];
  });
  toSign = toSign.substr(1) + commonParams.pn;
  var sign = Security.encryptMd5(toSign + ".security");
  commonParams.s = sign;
  var pkg = Base64.encode(JSON.stringify(commonParams));

  return { pkg };
};

/**
 * 请求拦截器 处理请求头
 */
request.interceptors.request.use(async (url, options) => {
  let headers = getHeader();
  const method = options.method.toUpperCase();
  if (method === 'POST' || method === 'PUT' || method === 'DELETE') {
    if (!(options.data instanceof FormData)) {
      headers = {
        Accept: 'application/json',
        'Content-Type': 'application/x-www-form-urlencoded; charset=utf-8',
        ...headers,
      };
      options.data = stringify(options.data);
    } else {
      headers = {
        Accept: 'application/json',
        ...headers,
      };
    }
  }

  return {
    url,
    options: { ...options, headers },
  };
});

/**
 * 请求结果拦截器
 */
request.interceptors.response.use(async (response, options) => {
  const res = await response.json();
  if (!res.success) {
    notification.error({
      message: '请求错误',
      description: res.msg,
    });

    // 令牌登录失败
    if (res.code === 'TOKEN_INVALID') {
      Cookie.remove('_xy_music', { path: '/' });
      // window.g_app._store.dispatch({
      //   type: 'login/logout',
      // });
      return res;
    }
  }
  return res;
});

export default request;
